require 'rho/rhocontroller'
require 'helpers/browser_helper'
require 'lib/authenticated'
require 'lib/rosp_http'
require 'json'

class SpotController < Rho::RhoController
  include BrowserHelper
  include Authenticated
  include RospHttp::Callback
  
  $camera_path = ""
  $back_url = ""
  
  # 未使用
  def index
    render :back => url_for(:controller => :Session, :action => :index)
  end
  
  # 周辺施設検索
  def search
    @latitude = GeoLocation.latitude
    @longitude = GeoLocation.longitude
    url_params = "?lat=#{@latitude}&lon=#{@longitude}"
    if @params["page"]
      url_params += "&page=#{@params["page"]}"
    end
    
    http = RospHttp.connect(
      :method => "get",
      :url => "#{Component.server_address}/spots/search.json#{url_params}",
      :back_url => url_for(:action => :search),
      :controller => self
    )
    body = http["body"]
    @spots = body["spots"]
    @page = body["page"]
    @spots = @spots.map{|s|s["spot"]}
    render :back => url_for(:action => :index)
  end
  
  # スポット全件
  def list
    @latitude = GeoLocation.latitude
    @longitude = GeoLocation.longitude
    if @params["page"]
      url_params = "?page=#{@params["page"]}"
    end
    
    http = RospHttp.connect(
      :method => "get",
      :url => "#{Component.server_address}/spots.json#{url_params}",
      :back_url => url_for(:action => :list),
      :controller => self
    )
    body = http["body"]
    @spots = body["spots"]
    @page = body["page"]
    @spots = @spots.map{|s|s["spot"]}
    render :back => url_for(:action => :index)
  end

  # GET /Spot/{1}
  def show
    if @params["back_url"]
      $back_url = url_for(:action => @params["back_url"])
    end
    $back_url = url_for(:action => :index) if $back_url == ""
    id = @params["id"].gsub(/[{|}]/, "")
    http = RospHttp.connect(
      :method => "get",
      :url => "#{Component.server_address}/spots/#{id}.json",
      :back_url => url_for(:action => :show, :id => id),
      :controller => self
    )
    @spot = http["body"]
    if @spot
      render :action => :show, :back => $back_url
    else
      redirect :action => :index
    end
  end
  
  def http_cancel
    RospHttp.cancel
    Alert.show_popup("中止しました。")
    redirect :action => :index
  end
  
  # 新規作成「現在地から取得」
  def set_geolocation_new
    @latitude = GeoLocation.latitude
    @longitude = GeoLocation.longitude
    @spot = {
      "latitude" => @latitude,
      "longitude" => @longitude
    }
    http = RospHttp.connect(
      :method => "get",
      :url => "#{Component.server_address}/spots/new.json",
      :back_url => url_for(:action => :set_geolocation_new),
      :controller => self
    )
    body = http["body"]
    @organizations = body["organizations"]
    if GeoLocation.known_position?
#    unless (@latitude =s= 0.0 && @longitude == 0.0)
      geo = Rho::AsyncHttp.get(
        :url => "http://maps.google.com/maps/geo?oe=utf-8&ll=#{@latitude},#{@longitude}&output=json"
      )
      if geo["status"] == "ok"# && geo["body"]["status"] == "200"
        geo_body = Rho::JSON.parse(geo["body"])
        place = geo_body["Placemark"].detect do |pl|
          pl["AddressDetails"]["Country"]["AdministrativeArea"].has_key?("Locality")
        end
        if place
          local = place["AddressDetails"]["Country"]["AdministrativeArea"]["Locality"]
          city = local["LocalityName"]
          address = local["DependentLocality"]["DependentLocalityName"]
          address += local["DependentLocality"]["Thoroughfare"]["ThoroughfareName"]

          @spot.merge!(
            "city" => city,
            "address" => address
          )
        else
          Alert.show_popup("現在地が正しく取得されませんでした。")
        end
      else
        Alert.show_popup("現在地が正しく取得されませんでした。")
      end
    else
      Alert.show_popup("現在地が正しく取得されませんでした。")
    end
    render :action => :new, :back => url_for(:action => :index)
  end
  
  # GET /Spot/new
  def new
    @latitude = GeoLocation.latitude
    @longitude = GeoLocation.longitude
    http = RospHttp.connect(
      :method => "get",
      :url => "#{Component.server_address}/spots/new.json",
      :back_url => url_for(:action => :new),
      :controller => self
    )
    @spot = {}
    body = http["body"]
    @organizations = body["organizations"]#.map{|o|o["organization"]}
    render :action => :new, :back => url_for(:action => :index)
  end

  # GET /Spot/{1}/edit
  def edit
    id = @params["id"].gsub(/[{|}]/, "")
    http = RospHttp.connect(
      :method => "get",
      :url => "#{Component.server_address}/spots/#{id}/edit.json",
      :back_url => url_for(:action => :edit, :id => id),
      :controller => self
    )
    body = http["body"]
    if body
      @spot = body["spot"]
      @organizations = body["organizations"]
      render :action => :edit, :back => url_for(:action => :index)
    else
      redirect :action => :index
    end
  end
  
  # 編集「現在地から取得」
  def set_geolocation_edit
    @latitude = GeoLocation.latitude
    @longitude = GeoLocation.longitude
    @latitude = 35.457152273713
    @longitude = 133.07755555533

    id = @params["id"].gsub(/[{|}]/, "")
    http = RospHttp.connect(
      :method => "get",
      :url => "#{Component.server_address}/spots/#{id}/edit.json",
      :back_url => url_for(:action => :edit, :id => id),
      :controller => self
    )
    body = http["body"]
    @spot = body["spot"].merge("latitude" => @latitude, "longitude" => @longitude)
    @organizations = body["organizations"]
#    if GeoLocation.known_position?
    unless (@latitude == 0.0 && @longitude == 0.0)
      geo = Rho::AsyncHttp.get(
        :url => "http://maps.google.com/maps/geo?oe=utf-8&ll=#{@latitude},#{@longitude}&output=json"
      )
      if geo["status"] == "ok"# && geo["body"]["status"] == "200"
        geo_body = Rho::JSON.parse(geo["body"])
        place = geo_body["Placemark"].detect do |pl|
          pl["AddressDetails"]["Country"]["AdministrativeArea"].has_key?("Locality")
        end
        if place
          local = place["AddressDetails"]["Country"]["AdministrativeArea"]["Locality"]
          city = local["LocalityName"]
          address = local["DependentLocality"]["DependentLocalityName"]
          address += local["DependentLocality"]["Thoroughfare"]["ThoroughfareName"]

          @spot.merge!(
            "city" => city,
            "address" => address
          )
        else
          Alert.show_popup("現在地が正しく取得されませんでした。")
        end
      else
        Alert.show_popup("現在地が正しく取得されませんでした。")
      end
    else
      Alert.show_popup("現在地が正しく取得されませんでした。")
    end
    render :action => :edit, :back => url_for(:action => :index)
  end
  
  def capture
    id = @params["id"].gsub(/[{|}]/, "")
    Camera::take_picture(url_for(:action => :capture_callback, :id => id))
    render 
  end
  
  def capture_callback
    @id = @params['id'].gsub(/[{|}]/, '')
    if @params["status"] == "ok"
      @image_url = Rho::RhoApplication::get_blob_path(@params["image_uri"])
      if File.exists?($camera_path)
        File.unlink($camera_path)
      end
      $camera_path = @image_url
      
      WebView.navigate(url_for(:action => :show_capture, :id => @id))
    else
      Alert.show_popup("画像の撮影に失敗しました")
      WebView.navigate(url_for(:action => :show,  :id => @id))
    end
  end
  
  def show_capture
    @id = @params['id'].gsub(/[{|}]/, '')
    @image_url = $camera_path
    render :back => url_for(:action => :show, :id => @id)
  end
  
  def upload_capture
    @id = @params['id'].gsub(/[{|}]/, '')
    image_path = $camera_path
    rosp_http = RospHttp.connect(
      :method => "upload_file",
      :multipart => [{
          :filename => image_path,
          :name =>  'file_path', :content_type =>  'application/octet-stream'}],
      :url => "#{Component.server_address}/spots/#{@id}/set_capture.json",
      :controller => self,
      :back_url => url_for(:action => :capture, :id => @id)
    )
    if rosp_http["status"] == "ok" && rosp_http["body"] == "OK"
      Alert.show_popup("撮影した画像をアップロードしました。")
    else
      Alert.show_popup("撮影した画像の送信に失敗しました。")
    end
    if File.exists?(Spot::IMAGE_PATH)
      File.unlink(Spot::IMAGE_PATH)
    end
    File.unlink(image_path) if File.exists?(image_path)
    redirect url_for(:action => :show, :id => @id)
#    redirect url_for(:action => :before_show, :id => @id)
  end
  
  # POST /Spot/create
  def create
    json = ::JSON.generate(:spot => @params["spot"])
    http = RospHttp.connect(
      :method => "post",
      :url => "#{Component.server_address}/spots.json?",
      :body => "json=#{json}",
      :back_url => url_for(:action => :new),
      :controller => self
    )
    if http["status"] == "ok" && http["body"]["status"] == "OK"
      body = http["body"]
      Alert.show_popup "スポットを登録しました。"
      redirect url_for(:action => :show, :id => body["spot"]["id"])
    else
      if http["body"].class.to_s == "Array"
        msg = http["body"]
        Alert.show_popup msg
        render :action => "new"
      else
        Alert.show_popup("データの作成に失敗しました。")
        redirect :action => :index
      end
    end
  end
  
  def show_image
    id = @params["id"].gsub(/[{|}]/, "")
    System.open_url("#{Component.server_address}/spots/#{id}/download")
    redirect :action => :show, :id => id
  end
  
  def remove_capture
    id = @params["id"].gsub(/[{|}]/, "")
    http = RospHttp.connect(
      :method => "post",
      :url => "#{Component.server_address}/spots/#{id}/remove_capture.json?",
      :back_url => url_for(:action => :show, :id => id),
      :controller => self
    )
    if http["status"] == "ok" && http["body"]["status"] == "OK"
      Alert.show_popup "写真の削除に成功しました。"
    else
      Alert.show_popup "写真の削除に失敗しました。"
    end
    redirect :action => :show, :id => id
  end
  
  # POST /Spot/{1}/update
  def update
    id = @params["id"].gsub(/[{|}]/, "")
    json = ::JSON.generate(:spot => @params["spot"])
    http = RospHttp.connect(
      :method => "post",
      :http_command => "PUT",
      :url => "#{Component.server_address}/spots/#{id}.json",
      :body => "json=#{json}",
      :back_url => url_for(:action => :edit),
      :controller => self
    )
    if http["status"] == "ok" && http["body"]["status"] == "OK"
      Alert.show_popup "スポットを編集しました。"
      redirect :action => :show, :id => id
    else
      if http["body"].class.to_s == "Array"
        msg = http["body"]
        Alert.show_popup msg
        render :action => "edit"
      else
        Alert.show_popup("データの更新に失敗しました。")
        redirect :action => :show, :id => id
      end
    end
  end

  # POST /Spot/{1}/delete
  def delete
    id = @params["id"].gsub(/[{|}]/, "")
    http = RospHttp.connect(
      :method => "post",
      :http_command => "DELETE",
      :url => "#{Component.server_address}/spots/#{id}.json?",
      :back_url => url_for(:action => :destory, :id => id),
      :controller => self
    )
    if http["status"] == "ok" && http["body"]["status"] == "OK"
      Alert.show_popup "スポットを削除しました。"
    else
      Alert.show_popup "スポットの削除に失敗しました。"
    end
    redirect :action => :index
  end
  
  def parents_category_list#parent_category_list
    @spot_id = @params["spot_id"]
    id = @params["id"].gsub(/[{|}]/, "")
    http = RospHttp.connect(
      :method => "get",
      :url => "#{Component.server_address}/spots/#{id}/category_list.json",
      :back_url => url_for(:action => :index),
      :controller => self
    )
    @categories = http["body"]["categories"]

  end
  
  def children_category_list
    @spot_id = @params["spot_id"]
    @id = @params["id"].gsub(/[{|}]/, "")
    http = RospHttp.connect(
      :method => "get",
      :url => "#{Component.server_address}/spots/#{@id}/category_list.json?spot_id=#{@spot_id}",
      :back_url => url_for(:action => :index),
      :controller => self
    )
    @categories = http["body"]["categories"]
    
    @categories_selected = []
    http["body"]["selected"].each do |c|
      @categories_selected << c["category_id"]
    end
  end
  
  def update_categories
    id = @params["spot_id"]
    json = ::JSON.generate(:categories_id => @params["categories"])
    http = RospHttp.connect(
      :method => "post",
      :url => "#{Component.server_address}/spots/#{id}/update_categories.json?parent_id=#{@params["parent_id"]}",
      :body => "json=#{json}",
      :back_url => url_for(:action => :edit),
      :controller => self
    )
    
    if http["status"] == "ok" && http["body"]["status"] == "OK"
      Alert.show_popup "カテゴリの登録に成功しました。"
    else
      Alert.show_popup "カテゴリの登録に失敗しました。"
    end
    redirect :action => :show, :id => id
  end
end
