require 'lib/authenticated'

#     RospHttp.connect
# Rho::AsyncHttpを独自に拡張したメソッド
# * self.connectメソッド -> 接続を行うメソッド
# * Callback::connect_callbackメソッド -> 接続後のコールバックメソッド。
# *  呼び出し元のコントローラでRospHttp::Callbackをincludeして使用する。
module RospHttp
  
  # authenticated system 
  AUTHENTICATED_SYSTEM = true
  
  # options 
  # * controller -> 呼び出しメソッドのインスタンス(self)を渡す必要がある。(必須)
  # * back_url -> 「再開」ボタンをおしたときに再度呼ぶアクションを指定する。
  def self.connect(options = {})
    parent_instance = options.delete(:controller)
    back_url = options.delete(:back_url) || Rho::RhoConfig.start_path
    callback_url = parent_instance.url_for(:action => :connect_callback, :query => {:back_url => back_url})
    
    if System.get_property("has_network")
      method = options.delete(:method)
      if AUTHENTICATED_SYSTEM
        options.merge!(:headers => {"Cookie" => parent_instance.login_cookie})
      end
      http = Rho::AsyncHttp.send(method.to_s.downcase, (options))
      unless http
        return {}
      end
      if http["status"] == "error"
        if http["http_error"] == "401"  # login failer
          Alert.show_popup("ログインし直してください。")
          parent_instance.redirect_login
          return {}
        else
          Alert.show_popup("データの取得に失敗しました。")
          return {}
        end
      else
        return http
      end
    else
      # ネットワークに接続されていないとき
      Alert.show_popup(
        :message => "ネットワークに接続してください。",
        :buttons => [
          {:id => "res", :title => "再開"},
          {:id => "stop", :title => "中止"},
        ],
        :callback => callback_url
      )
    end
  end
  
  def self.cancel
    Rho::AsyncHttp.cancel()
  end

  # ネットワークに接続されていない場合に再度
  module Callback
    def connect_callback
      case @params["button_id"]
      when "res" # 再開
        # 再度アクションを呼ぶ。
        WebView.navigate(@params["back_url"])
      when "stop" # 中止
        # ホームに戻る。
        WebView.navigate(Rho::RhoConfig.start_path)
      end
    end
  end
end